#include <Wire.h> 
#include <LiquidCrystal_I2C.h>             // von https://github.com/marcoschwartz/LiquidCrystal_I2C
#include "PushButton.h"                    // ist eine eigene Klasse, um Buttons zu entprellen (ohne Blockierung)

// Achtung! Fuer die Baudrate habe ich hier nur ein "word" reserviert. 
// Das ist beim Nano nur ein 16-Bit-Wert (max. 65535), bei groesseren Werten muss "long" (32 Bit) verwendet werden!
const word Baudrate = 19200;               // Baudrate fuer die serielle Schnittstelle (muss mit der im AutoIt-Programm uebereinstimmen)
const byte ButtonPin = 2;                  // PushButton am Pin D2 (vom Nano)
const byte PiezoPin = 4;                   // Piezosummer am Pin D4 (vom Nano)
const byte LEDintern =  13;                // interne LED zum blinken zusammen mit den Pieptoenen
// Achtung! Fuer die beiden folgenden Zeiten habe ich nur ein "byte" reserviert.
// Ein Byte kann Werte bis 255 annehmen. Sind laengere Zeiten gewuenscht, muss "word" verwendet werden! 
const byte PiezoToneTime = 150;            // Piepdauer des Piezo (in ms)
const byte PiezoPauseTime = 100;           // Pausendauer zwischen den Pieptoenen (in ms)
const byte PiezoToneCount = 3;             // Anzahl der Pieptoene festlegen

byte row = 0;                              // Zeilennummer des LCD
byte PiezoCount = 0;                       // Zaehler fuer die Pieptoene
bool PiezoPause = false, NewMail = false;  // Piezo nicht im Pausemodus und keine "Neue Nachricht"
unsigned long PiezoTime = 0;               // Variable zum merken, der Piep- und Pausendauer (muss "unsigned long" sein, weil millis() gespeichert wird)

LiquidCrystal_I2C lcd(0x27,20,4);          // LCD-Objekt erstellen (I2C-Adresse: 0x27, 20 Zeichen/Zeile und 4 Zeilen)
PushButton PBtn;                           // PushButton-Objekt erstellen

void setup(){
  pinMode(PiezoPin, OUTPUT);               // Piezo-Port auf Ausgang schalten
  digitalWrite(PiezoPin, LOW);             // und auf LOW legen (aus)
  pinMode(LEDintern, OUTPUT);              // interne LED auf Ausgang schalten
  digitalWrite(LEDintern, LOW);            // und auf LOW legen (aus)
  Serial.begin(Baudrate);                  // Serielle Schnittstelle konfigurieren: Baudrate und sonstige Parameter (Default: 8N1)
  lcd.init();                              // LCD initialisieren
  lcd.clear();                             // LCD leeren
  PushButton::setdbTime(50);               // Entprellzeit fuer den Taster festlegen (50 ms)
  PBtn.setPin(ButtonPin);                  // Pin vom Nano (siehe oben) fuer den Taster festlegen (wird intern auf PULLUP gelegt, Taster also gegen GND)
  PBtn.setState(0,0);                      // als Taster konfigurieren, ohne Wiederholung
}

void loop() {
  if (NewMail && PBtn.KeyDown()) {         // wenn das LCD eingeschaltet ist und der Pushbutton betaetigt wurde
    lcd.noDisplay();                       // das LCD ausschalten
    lcd.noBacklight();                     // die Hintergrundbeleuchtung ausschalten
    row = 0;                               // Zeilennummer auf 0 setzen
    NewMail = false;                       // "Neue Nachricht"-Markierung zuruecksetzen
  }
  if (Serial.available() > 0) {            // wenn neue Zeichen im seriellen Puffer eingetroffen sind
    char ReadChar = Serial.read();         // ein Zeichen holen
    while (ReadChar != 2) {                // auf STX (ASCII: 2) als "Uebertragungsanfang" warten
      ReadChar = Serial.read();            // evtl. vorhandene ungueltige Zeichen aus dem Puffer auslesen und verwerfen
    }
    lcd.clear();                           // das Display leeren
    lcd.setCursor(0,row);                  // den Cursor auf die aktuelle Zeile setzen
    while (Serial.available() > 0) {       // solange sich noch Zeichen im Puffer befinden
      ReadChar = Serial.read();            // ein Zeichen aus dem seriellen Puffer holen
      switch (ReadChar) {
        case 4:                            // wenn EOT (ASCII: 4) als "Uebertragungsende" empfangen wurde
          lcd.display();                   // das LCD einschalten
          lcd.backlight();                 // die Hintergrundbeleuchtung des LCD einschalten
          NewMail = true;                  // "Neue Nachricht"-Markierung setzen
          Serial.write("ok");              // Bestaetigung "ok" zurueck an AutoIt senden
          if (!PiezoToneCount) break;      // wenn kein Piepton gewuenscht wird (PiezoToneCount = 0), dann zum Ende von Switch
          digitalWrite(LEDintern, HIGH);   // Interne LED einschalten
          digitalWrite(PiezoPin, HIGH);    // Piezo einschalten
          PiezoTime = millis();            // Anfang der Piepdauer merken
          PiezoCount = PiezoToneCount;     // Anzahl der Pieptoene festlegen
          PiezoPause = false;              // Piezo nicht im Pausemodus
          break;
        case 10:                           // wenn LF (ASCII: 10) als "neue Zeile" empfangen wurde
          row++;                           // Zeilennummer um eins erhoehen
          if (row >= 4) row = 0;           // wenn groesser/gleich 4, dann auf 0 setzen 
          lcd.setCursor(0,row);            // den Cursor auf die aktuelle Zeile setzen
          break;
        default:                           // alle anderen empfangenen Zeichen
          lcd.print(ReadChar);             // an das LCD senden
      }
    }
  }
  if (PiezoCount) {                        // Wenn Anzahl der Pieptoene > 0
    if (!PiezoPause && (millis() - PiezoTime > PiezoToneTime)) {   // wenn nicht im Pausemodus und die Piepdauer ueberschritten wurde 
      digitalWrite(LEDintern, LOW);        // Interne LED ausschalten
      digitalWrite(PiezoPin, LOW);         // Piezo ausschalten
      PiezoCount--;                        // Piepzaehler um eins verringern
      PiezoPause = true;                   // Pausemodus beginnen
      PiezoTime = millis();                // Anfang der Pausendauer merken
    }
    if (PiezoPause && (millis() - PiezoTime > PiezoPauseTime)) {   // wenn im Pausemodus und die Pausedauer ueberschritten wurde
      digitalWrite(LEDintern, HIGH);       // Interne LED einschalten
      digitalWrite(PiezoPin, HIGH);        // Piezo einschalten
      PiezoPause = false;                  // Pausemodus beenden
      PiezoTime = millis();                // Anfang der Piepdauer merken
    }
  }
}
